#!/usr/bin/python


# TODO: add "skip"

program_name = "bkgd"
program_version = "0.4"

#
# Config file handling, and defaults...
#

############################################################
# Config class:
#
def is_config(obj):
  return hasattr(obj,'is_config') and obj.is_config

class config:
  is_config = 1

  def __init__(self):
    import os
    self.options = {}
    self.usrfilename = "%s/.%s.conf" % (os.environ["HOME"], program_name)
    self.globalfilename = "/etc/%s/%s.conf" % (program_name, program_name)
    self.localfilename = "/usr/local/etc/%s/%s.conf" % (program_name, program_name)
    self.namespace = {
      "options" : self.options,
      }

  def load(self):
    import os.path
    for name in [self.globalfilename, self.localfilename, self.usrfilename]:
      if os.path.exists(name):
        self.parse(name)

  def parse(self, filename):
    #print "parsing %s..." % filename
    
    try:
      execfile(filename, self.namespace)
    except:
      print "Error in config file %s" % filename
      import traceback, sys
      foo = traceback.format_exception(sys.exc_type,
                                       sys.exc_value,
                                       sys.exc_traceback)
      for item in foo:
        item = item[:-1]
        print item
      print "Press Enter to continue..."
      raw_input()
      


######################################################################
#
class imglist:

  def __init__(self, cfg=None):
    self.cfg = cfg
    self.files = []
    self.filters = []

  ##################################################
  #
  def set_random(self):
    import whrandom, string, os

    if not self.files  and  not self.filters:
      print """It appears you have not yet set up an image repository for bkgd.
You must do this before bkgd can do anything.

Creating a bkgd image library
-----------------------------

1.  Decide where to put your background images.  $HOME/backgrounds/ is the
    recommended place to put them.

2.  Configure bkgd to use this image library.  Create a file called
    ".bkgd.conf" in your home directory, and add a line to specify where
    bkgd should look for images.  You can specify more than one directory
    if you like:

        options["path"] = ["/home/john/backgrounds",
                           "/usr/share/backgrounds"]

3.  Find some images to use for wallpaper.  There are thousands of sites
    online with a variety of interesting images.  Put all of your images
    in the directory you have chosen.

4.  Sort your backgrounds by desired display method.  Supported methods are
    "tiled", "scaled", "centered", and "skip".  To sort them, put tileable
    images in a subdirectory called "tile" or "tiled".  Do the same with the
    other methods.  The "skip" images will never be used.

5.  You may also need to add configuration options to tell bkgd which image
    viewer you want to use.  There are preset settings for several,
    including qiv, xv, and Esetroot.  To use one of these, add a line to
    your .bkgd.conf, which looks like this:

	preset_qiv()

    To use a non-supported image viewer, you must set a few options:

	options["viewer"]   = "Esetroot"
	options["tile"]    = ""
	options["scale"]   = "-scale"
	options["center"] = "-center"
		
Enjoy!
"""
      return

    if not self.files:
      print 'No images found!'
      return

    img = whrandom.choice(self.files)

    method = 'tile'
    for m in ['scale', 'center', 'tile']:
      if 0 <= string.find(img, m):
        method = m

    cmd = "%s %s '%s'" % \
          (self.cfg.options["viewer"],
           self.cfg.options[method],
           img)
    #print cmd
    print img
    os.system(cmd)
    
  ##################################################
  #
  def add_filter(self, pattern):
    self.filters.append(pattern)

  ##################################################
  #
  def load(self, path):
    import os
    os.path.walk(path, self._walker, None)

  ##################################################
  #
  def load_all(self):
    for path in self.cfg.options["path"]:
      self.load(path)

  ##################################################
  #
  def _walker(self, data, path, files):
    #print path
    #print files

    import string, os

    for f in files:
      fullpath = "%s/%s" % (path, f)

      # Skip directories and such
      if not os.path.isfile(fullpath):
        continue

      # Skip files with "skip" in them.
      if 0 <= string.find(fullpath, "skip"):
        continue

      if not self.filters:
        self.files.append(fullpath)
      else:
        found = 0
        #print "%s..." % fullpath
        for fl in self.filters:
          if 0 <= string.find(fullpath, fl):
            found = 1
        if found:
          self.files.append(fullpath)
          #print "found"


######################################################################
#
def run(args):
  import os

  #print "run(%s)" % (args)
  cfg = config()
  cfg.load()

  imgs = imglist(cfg)

  for a in args:
    imgs.add_filter(a)

  imgs.load_all()

  imgs.set_random()


######################################################################
#
if __name__ == "__main__":
  import sys
  run(sys.argv[1:])
#else:
#  print "__name__ == %s" % __name__
